@@ -1,6 +1,4 @@
-/* acconfig.h for BEAST - Bedevilled Audio System
- * here we list all symbols, which get introduced by configure.in
- */
+/* acconfig.h for BEAST - here we list all symbols, which get introduced by configure.in */
 
 #include "PKG_paths.h"
 
@@ -9,10 +7,12 @@
 #undef  BST_ARCH_NAME
 #undef  BST_VERSION
 #undef  BST_VERSION_HINT
+#undef  BST_VERSION_STABLE
 #undef  BST_DFL_KEYTABLE
 #undef  BST_WITH_XKB
 #undef  BST_WITH_GDK_PIXBUF
 #undef  BSE_PCM_DEVICE_CONF_ALSA
 #undef  BSE_PCM_DEVICE_CONF_OSS
+#undef  BSE_MIDI_DEVICE_CONF_OSS
 
 @BOTTOM@
