@@ -2,6 +2,8 @@
 /* Generated data (by glib-mkenums) */
 
 
+GType bst__type_id__BstFileDialogMode = 0;
+
 GType bst__type_id__BstMarkerType = 0;
 
 GType bst__type_id__BstChoiceFlags = 0;
@@ -12,7 +14,7 @@ GType bst__type_id__BstQSamplerDrawMode = 0;
 
 GType bst__type_id__BstRadioToolsFlags = 0;
 
-GType bst__type_id__BstOps = 0;
+GType bst__type_id__BstActionRegions = 0;
 
 GType bst__type_id__BstQuantizationType = 0;
 
@@ -22,6 +24,7 @@ GType bst__type_id__BstQuantizationType = 0;
 /*-------- begin ../sfi/sfidl generated code --------*/
 
 
+#include <string.h>
 BstGConfig*
 bst_gconfig_new (void)
 {
@@ -32,10 +35,10 @@ bst_gconfig_new (void)
 BstGConfig*
 bst_gconfig_copy_shallow (BstGConfig* rec)
 {
-  BstGConfig* rec_copy = NULL;
+  BstGConfig* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bst_gconfig_new ();
+  rec_copy = g_new0 (BstGConfig, 1);
   rec_copy->rc_version = g_strdup (rec->rc_version);
   rec_copy->snet_anti_aliased = rec->snet_anti_aliased;
   rec_copy->snet_edit_fallback = rec->snet_edit_fallback;
