@@ -91,8 +91,8 @@ static void
 bst_param_view_destroy_contents (BstParamView *param_view)
 {
   gtk_container_foreach (GTK_CONTAINER (param_view), (GtkCallback) gtk_widget_destroy, NULL);
-  g_slist_free (param_view->bparams);
-  param_view->bparams = NULL;
+  while (param_view->bparams)
+    bst_param_destroy (g_slist_pop_head (&param_view->bparams));
 }
 
 static void
@@ -240,7 +240,8 @@ bst_param_view_rebuild (BstParamView *self)
 	GParamSpec *pspec = bse_proxy_get_pspec (self->item, pstrings[i]);
 	const gchar *param_group = sfi_pspec_get_group (pspec);
 
-	if (sfi_pspec_test_hint (pspec, SFI_PARAM_SERVE_GUI) && (pspec->flags & G_PARAM_READABLE))
+	if (sfi_pspec_test_hint (pspec, SFI_PARAM_SERVE_GUI) && (pspec->flags & G_PARAM_READABLE) &&
+            ((pspec->flags & G_PARAM_WRITABLE) || BST_DVL_HINTS))
 	  {
 	    BstParam *bparam = bst_param_proxy_create (pspec, FALSE, NULL, self->item);
 	    if (param_group)
