@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 2002 Tim Janik
+ * Copyright (C) 2002-2003 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #include "bstpianorollctrl.h"
+#include "bsteventrollctrl.h"
 
 
 
@@ -38,7 +39,9 @@ bst_piano_roll_controller_set_clipboard (BsePartNoteSeq *pseq)
 {
   if (clipboard_pseq)
     bse_part_note_seq_free (clipboard_pseq);
-  clipboard_pseq = bse_part_note_seq_copy_shallow (pseq);
+  clipboard_pseq = pseq && pseq->n_pnotes ? bse_part_note_seq_copy_shallow (pseq) : NULL;
+  if (clipboard_pseq)
+    bst_event_roll_controller_set_clipboard (NULL);
 }
 
 BsePartNoteSeq*
@@ -131,11 +134,13 @@ bst_piano_roll_controller_clear (BstPianoRollController *self)
 
   proxy = self->proll->proxy;
   pseq = bse_part_list_selected_notes (proxy);
+  bse_item_group_undo (proxy, "Clear Selection");
   for (i = 0; i < pseq->n_pnotes; i++)
     {
       BsePartNote *pnote = pseq->pnotes[i];
       bse_part_delete_event (proxy, pnote->id);
     }
+  bse_item_ungroup_undo (proxy);
 }
 
 void
@@ -149,25 +154,28 @@ bst_piano_roll_controller_cut (BstPianoRollController *self)
 
   proxy = self->proll->proxy;
   pseq = bse_part_list_selected_notes (proxy);
+  bse_item_group_undo (proxy, "Cut Selection");
   for (i = 0; i < pseq->n_pnotes; i++)
     {
       BsePartNote *pnote = pseq->pnotes[i];
       bse_part_delete_event (proxy, pnote->id);
     }
   bst_piano_roll_controller_set_clipboard (pseq);
+  bse_item_ungroup_undo (proxy);
 }
 
-void
+gboolean
 bst_piano_roll_controller_copy (BstPianoRollController *self)
 {
   BsePartNoteSeq *pseq;
   SfiProxy proxy;
 
-  g_return_if_fail (self != NULL);
+  g_return_val_if_fail (self != NULL, FALSE);
 
   proxy = self->proll->proxy;
   pseq = bse_part_list_selected_notes (proxy);
   bst_piano_roll_controller_set_clipboard (pseq);
+  return pseq && pseq->n_pnotes;
 }
 
 void
@@ -179,21 +187,23 @@ bst_piano_roll_controller_paste (BstPianoRollController *self)
   g_return_if_fail (self != NULL);
 
   proxy = self->proll->proxy;
-  bse_part_deselect_rectangle (proxy, 0, self->proll->max_ticks, self->proll->min_note, self->proll->max_note);
   pseq = bst_piano_roll_controller_get_clipboard ();
   if (pseq)
     {
-      guint i, ptick, ctick = self->proll->max_ticks;
+      guint i, paste_tick, ctick = self->proll->max_ticks;
       gint cnote = 0;
-      gint pnote;
-      bst_piano_roll_get_paste_pos (self->proll, &ptick, &pnote);
+      gint paste_note;
+      bse_item_group_undo (proxy, "Paste Clipboard");
+      bse_part_deselect_notes (proxy, 0, self->proll->max_ticks, self->proll->min_note, self->proll->max_note);
+      bst_piano_roll_get_paste_pos (self->proll, &paste_tick, &paste_note);
+      paste_tick = bst_piano_roll_quantize (self->proll, paste_tick);
       for (i = 0; i < pseq->n_pnotes; i++)
 	{
 	  BsePartNote *pnote = pseq->pnotes[i];
 	  ctick = MIN (ctick, pnote->tick);
 	  cnote = MAX (cnote, pnote->note);
 	}
-      cnote = pnote - cnote;
+      cnote = paste_note - cnote;
       for (i = 0; i < pseq->n_pnotes; i++)
 	{
 	  BsePartNote *pnote = pseq->pnotes[i];
@@ -203,18 +213,25 @@ bst_piano_roll_controller_paste (BstPianoRollController *self)
 	  if (note >= 0)
 	    {
 	      id = bse_part_insert_note (proxy,
-					 pnote->tick - ctick + ptick,
+					 pnote->tick - ctick + paste_tick,
 					 pnote->duration,
 					 note,
 					 pnote->fine_tune,
 					 pnote->velocity);
-	      if (id)
-		bse_part_select_event (proxy, id);
-	    }
+              bse_part_select_event (proxy, id);
+            }
 	}
+      bse_item_ungroup_undo (proxy);
     }
 }
 
+gboolean
+bst_piano_roll_controler_clipboard_full (BstPianoRollController *self)
+{
+  BsePartNoteSeq *pseq = bst_piano_roll_controller_get_clipboard ();
+  return pseq && pseq->n_pnotes;
+}
+
 static void
 controller_update_cursor (BstPianoRollController *self,
 			  BstPianoRollTool        tool)
@@ -306,6 +323,7 @@ move_group_motion (BstPianoRollController *self,
   delta_note = old_note;
   delta_tick -= new_tick;
   delta_note -= new_note;
+  bse_item_group_undo (part, "Move Selection");
   for (i = 0; i < self->sel_pseq->n_pnotes; i++)
     {
       BsePartNote *pnote = self->sel_pseq->pnotes[i];
@@ -324,6 +342,7 @@ move_group_motion (BstPianoRollController *self,
       bse_part_note_seq_free (self->sel_pseq);
       self->sel_pseq = NULL;
     }
+  bse_item_ungroup_undo (part);
 }
 
 static void
@@ -331,7 +350,7 @@ move_motion (BstPianoRollController *self,
 	     BstPianoRollDrag       *drag)
 {
   SfiProxy part = self->proll->proxy;
-  guint new_tick;
+  gint new_tick;
   gboolean note_changed;
 
   if (self->sel_pseq)
@@ -347,6 +366,7 @@ move_motion (BstPianoRollController *self,
       !check_hoverlap (part, new_tick, self->obj_duration, drag->current_note,
 		       self->obj_tick, note_changed ? 0 : self->obj_duration))
     {
+      bse_item_group_undo (part, "Move Note");
       if (bse_part_delete_event (part, self->obj_id) != BSE_ERROR_NONE)
 	drag->state = BST_DRAG_ERROR;
       else
@@ -358,6 +378,7 @@ move_motion (BstPianoRollController *self,
 	  if (!self->obj_id)
 	    drag->state = BST_DRAG_ERROR;
 	}
+      bse_item_ungroup_undo (part);
     }
 }
 
@@ -416,6 +437,7 @@ resize_motion (BstPianoRollController *self,
       !check_hoverlap (part, new_tick, new_duration, self->obj_note,
 		       self->obj_tick, self->obj_duration))
     {
+      bse_item_group_undo (part, "Resize Note");
       if (self->obj_id)
 	{
 	  BseErrorType error = bse_part_delete_event (part, self->obj_id);
@@ -432,6 +454,7 @@ resize_motion (BstPianoRollController *self,
 	  if (!self->obj_id)
 	    drag->state = BST_DRAG_ERROR;
 	}
+      bse_item_ungroup_undo (part);
     }
 }
 
@@ -502,7 +525,7 @@ select_motion (BstPianoRollController *self,
   bst_piano_roll_set_view_selection (drag->proll, start_tick, end_tick - start_tick, min_note, max_note);
   if (drag->type == BST_DRAG_DONE)
     {
-      bse_part_select_rectangle_exclusive (part, start_tick, end_tick - start_tick, min_note, max_note);
+      bse_part_select_notes_exclusive (part, start_tick, end_tick - start_tick, min_note, max_note);
       bst_piano_roll_set_view_selection (drag->proll, 0, 0, 0, 0);
     }
 }
@@ -537,8 +560,8 @@ vselect_motion (BstPianoRollController *self,
 				     drag->proll->min_note, drag->proll->max_note);
   if (drag->type == BST_DRAG_DONE)
     {
-      bse_part_select_rectangle_exclusive (part, start_tick, end_tick - start_tick,
-					   drag->proll->min_note, drag->proll->max_note);
+      bse_part_select_notes_exclusive (part, start_tick, end_tick - start_tick,
+                                       drag->proll->min_note, drag->proll->max_note);
       bst_piano_roll_set_view_selection (drag->proll, 0, 0, 0, 0);
     }
 }
@@ -680,7 +703,7 @@ controller_piano_drag (BstPianoRollController *self,
 	  error = bse_project_activate (project);
 	  self->obj_note = drag->current_note;
 	  if (error == BSE_ERROR_NONE)
-	    bse_song_synthesize_note (song, track, 384, self->obj_note, 0, 1.0);
+	    bse_song_synthesize_note (song, track, 384 * 4, self->obj_note, 0, 1.0);
 	  bst_status_eprintf (error, "Play note");
 	  drag->state = BST_DRAG_CONTINUE;
 	}
