@@ -72,8 +72,9 @@ void			bst_piano_roll_controller_set_clipboard (BsePartNoteSeq		*pseq);
 BsePartNoteSeq*		bst_piano_roll_controller_get_clipboard	(void);
 void			bst_piano_roll_controller_clear		(BstPianoRollController	*self);
 void			bst_piano_roll_controller_cut		(BstPianoRollController	*self);
-void			bst_piano_roll_controller_copy		(BstPianoRollController	*self);
+gboolean		bst_piano_roll_controller_copy		(BstPianoRollController	*self);
 void			bst_piano_roll_controller_paste		(BstPianoRollController	*self);
+gboolean                bst_piano_roll_controler_clipboard_full (BstPianoRollController *self);
 
 
 G_END_DECLS
