@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 2002 Tim Janik
+ * Copyright (C) 2002-2003 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,12 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #include "bsttrackview.h"
+#include "bstactivatable.h"
+#include "bstparam.h"
+#include "bsttracksynthdialog.h"
 
 #include <stdlib.h> /* strtol */
+#include <string.h>
 
 #define	SCROLLBAR_SPACING (3) /* from gtkscrolledwindow.c:DEFAULT_SCROLLBAR_SPACING */
 
@@ -25,10 +29,10 @@
 static void	bst_track_view_class_init	(BstTrackViewClass	*klass);
 static void	bst_track_view_init		(BstTrackView		*self);
 static void	bst_track_view_finalize		(GObject		*object);
-static void	bst_track_view_operate		(BstItemView		*item_view,
-						 BstOps			 op);
-static gboolean	bst_track_view_can_operate	(BstItemView		*item_view,
-						 BstOps			 op);
+static void     bst_track_view_activate         (BstActivatable         *activatable,
+                                                 gulong                  action);
+static gboolean bst_track_view_can_activate     (BstActivatable         *activatable,
+                                                 gulong                  action);
 static void	bst_track_view_create_tree	(BstItemView		*iview);
 static void     track_view_set_container        (BstItemView            *self,
 						 SfiProxy                new_container);
@@ -40,10 +44,12 @@ static void	track_view_unlisten_on		(BstItemView		*iview,
 
 /* --- columns --- */
 enum {
+  COL_SEQID,
   COL_NAME,
   COL_MUTE,
   COL_VOICES,
   COL_SYNTH,
+  COL_POST_SYNTH,
   COL_BLURB,
   N_COLS
 };
@@ -51,9 +57,9 @@ enum {
 
 /* --- track ops --- */
 static BstItemViewOp track_view_ops[] = {
-  { "Add",		BST_OP_TRACK_ADD,	BST_STOCK_TRACK,
+  { "Add",		BST_ACTION_ADD_TRACK,   	BST_STOCK_TRACK,
     "Add a new track to this song" },
-  { "Delete",		BST_OP_TRACK_DELETE,	BST_STOCK_TRASHCAN,
+  { "Delete",		BST_ACTION_DELETE_TRACK,	BST_STOCK_TRASHCAN,
     "Delete the currently selected track" },
 };
 static guint n_track_view_ops = sizeof (track_view_ops) / sizeof (track_view_ops[0]);
@@ -64,29 +70,30 @@ static gpointer parent_class = NULL;
 
 
 /* --- functions --- */
-GtkType
+GType
 bst_track_view_get_type (void)
 {
-  static GtkType track_view_type = 0;
-  
-  if (!track_view_type)
+  static GType type = 0;
+  if (!type)
     {
-      GtkTypeInfo track_view_info =
-      {
-	"BstTrackView",
-	sizeof (BstTrackView),
-	sizeof (BstTrackViewClass),
-	(GtkClassInitFunc) bst_track_view_class_init,
-	(GtkObjectInitFunc) bst_track_view_init,
-	/* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-	(GtkClassInitFunc) NULL,
+      static const GTypeInfo type_info = {
+        sizeof (BstTrackViewClass),
+        (GBaseInitFunc) NULL,
+        (GBaseFinalizeFunc) NULL,
+        (GClassInitFunc) bst_track_view_class_init,
+        NULL,   /* class_finalize */
+        NULL,   /* class_data */
+        sizeof (BstTrackView),
+        0,      /* n_preallocs */
+        (GInstanceInitFunc) bst_track_view_init,
       };
-      
-      track_view_type = gtk_type_unique (BST_TYPE_ITEM_VIEW, &track_view_info);
+      type = g_type_register_static (BST_TYPE_ITEM_VIEW, "BstTrackView", &type_info, 0);
+      bst_type_implement_activatable (type,
+                                      bst_track_view_activate,
+                                      bst_track_view_can_activate,
+                                      NULL);
     }
-  
-  return track_view_type;
+  return type;
 }
 
 static void
@@ -103,22 +110,17 @@ bst_track_view_class_init (BstTrackViewClass *class)
   item_view_class->ops = track_view_ops;
   item_view_class->horizontal_ops = TRUE;
   item_view_class->show_properties = FALSE;
-  item_view_class->operate = bst_track_view_operate;
-  item_view_class->can_operate = bst_track_view_can_operate;
   item_view_class->create_tree = bst_track_view_create_tree;
   item_view_class->set_container = track_view_set_container;
   item_view_class->listen_on = track_view_listen_on;
   item_view_class->unlisten_on = track_view_unlisten_on;
+
+  item_view_class->item_type = "BseTrack";
 }
 
 static void
 bst_track_view_init (BstTrackView *self)
 {
-  BstItemView *iview = BST_ITEM_VIEW (self);
-
-  /* configure item view for tracks */
-  iview->item_type = "BseTrack";
-  bst_item_view_set_id_format (iview, "%02X");
 }
 
 static void
@@ -159,41 +161,165 @@ track_view_fill_value (BstItemView *iview,
 		       guint        row,
 		       GValue      *value)
 {
+  BstTrackView *self = BST_TRACK_VIEW (iview);
   guint seqid = row + 1;
   switch (column)
     {
       const gchar *string;
       gboolean vbool;
       SfiInt vint;
-      SfiProxy item, snet;
+      SfiProxy item, snet, wave;
+    case COL_SEQID:
+      sfi_value_take_string (value, g_strdup_printf ("%03d", seqid));
+      break;
     case COL_NAME:
-      item = bse_container_get_item (iview->container, iview->item_type, seqid);
+      item = bse_container_get_item (iview->container, BST_ITEM_VIEW_GET_CLASS (self)->item_type, seqid);
       g_value_set_string (value, bse_item_get_name (item));
       break;
     case COL_MUTE:
-      item = bse_container_get_item (iview->container, iview->item_type, seqid);
+      item = bse_container_get_item (iview->container, BST_ITEM_VIEW_GET_CLASS (self)->item_type, seqid);
       bse_proxy_get (item, "muted", &vbool, NULL);
       g_value_set_boolean (value, !vbool);
       break;
     case COL_VOICES:
-      item = bse_container_get_item (iview->container, iview->item_type, seqid);
+      item = bse_container_get_item (iview->container, BST_ITEM_VIEW_GET_CLASS (self)->item_type, seqid);
       bse_proxy_get (item, "n_voices", &vint, NULL);
       sfi_value_take_string (value, g_strdup_printf ("%2d", vint));
       break;
     case COL_SYNTH:
-      item = bse_container_get_item (iview->container, iview->item_type, seqid);
+      item = bse_container_get_item (iview->container, BST_ITEM_VIEW_GET_CLASS (self)->item_type, seqid);
       snet = 0;
-      bse_proxy_get (item, "snet", &snet, NULL);
+      bse_proxy_get (item, "snet", &snet, "wave", &wave, NULL);
+      g_value_set_string (value, snet || wave ? bse_item_get_name (snet ? snet : wave) : "");
+      break;
+    case COL_POST_SYNTH:
+      item = bse_container_get_item (iview->container, BST_ITEM_VIEW_GET_CLASS (self)->item_type, seqid);
+      snet = 0;
+      bse_proxy_get (item, "pnet", &snet, NULL);
       g_value_set_string (value, snet ? bse_item_get_name (snet) : "");
       break;
     case COL_BLURB:
-      item = bse_container_get_item (iview->container, iview->item_type, seqid);
+      item = bse_container_get_item (iview->container, BST_ITEM_VIEW_GET_CLASS (self)->item_type, seqid);
       bse_proxy_get (item, "blurb", &string, NULL);
       g_value_set_string (value, string ? string : "");
       break;
     }
 }
 
+static void
+track_view_synth_edited (BstTrackView *self,
+			 const gchar  *strpath,
+			 const gchar  *text)
+{
+  g_return_if_fail (BST_IS_TRACK_VIEW (self));
+
+  if (strpath)
+    {
+      gint row = gxk_tree_spath_index0 (strpath);
+      SfiProxy item = bst_item_view_get_proxy (BST_ITEM_VIEW (self), row);
+      if (text)
+	{
+	  SfiProxy proxy = 0;
+	  GSList *slist = NULL;
+	  /* list possible snet/wave candidates */
+	  slist = g_slist_append (slist, bse_item_list_proxies (item, "snet"));
+	  slist = g_slist_append (slist, bse_item_list_proxies (item, "wave"));
+	  /* find best match */
+	  proxy = bst_proxy_seq_list_match (slist, text);
+	  g_slist_free (slist);
+	  if (proxy && BSE_IS_SNET (proxy))
+	    bse_proxy_set (item, "snet", proxy, NULL);
+	  else if (proxy && BSE_IS_WAVE (proxy))
+	    bse_proxy_set (item, "wave", proxy, NULL);
+	  else
+	    bse_proxy_set (item, "snet", 0, "wave", 0, NULL);
+	}
+      else
+	bse_proxy_set (item, "snet", 0, "wave", 0, NULL);
+    }
+}
+
+static void
+track_view_post_synth_edited (BstTrackView *self,
+                              const gchar  *strpath,
+                              const gchar  *text)
+{
+  g_return_if_fail (BST_IS_TRACK_VIEW (self));
+
+  if (strpath)
+    {
+      gint row = gxk_tree_spath_index0 (strpath);
+      SfiProxy item = bst_item_view_get_proxy (BST_ITEM_VIEW (self), row);
+      if (text)
+	{
+	  SfiProxy proxy = 0;
+	  GSList *slist = NULL;
+	  /* list possible snet candidates */
+	  slist = g_slist_append (slist, bse_item_list_proxies (item, "pnet"));
+	  /* find best match */
+	  proxy = bst_proxy_seq_list_match (slist, text);
+	  g_slist_free (slist);
+	  if (proxy && BSE_IS_SNET (proxy))
+	    bse_proxy_set (item, "pnet", proxy, NULL);
+	  else
+	    bse_proxy_set (item, "pnet", 0, NULL);
+	}
+      else
+	bse_proxy_set (item, "pnet", 0, NULL);
+    }
+}
+
+static void
+track_view_synth_popup_cb (GxkCellRendererPopup *pcell,
+                           SfiProxy              proxy,
+                           BstTrackSynthDialog  *tsdialog)
+{
+  gxk_cell_renderer_enter (pcell,
+                           proxy ? bse_item_get_uname_path (proxy) : "",
+                           FALSE,
+                           proxy == 0);
+}
+
+static void
+track_view_synth_popup (BstTrackView         *self,
+			const gchar          *strpath,
+			const gchar          *text,
+			GxkCellRendererPopup *pcell)
+{
+  g_return_if_fail (BST_IS_TRACK_VIEW (self));
+
+  if (strpath)
+    {
+      gint row = gxk_tree_spath_index0 (strpath);
+      SfiProxy item = bst_item_view_get_proxy (BST_ITEM_VIEW (self), row);
+      BseProxySeq *pseq = bse_item_list_proxies (item, "snet");
+      GtkWidget *dialog = bst_track_synth_dialog_popup (self, item,
+                                                        pseq,
+                                                        bse_project_get_wave_repo (bse_item_get_project (item)),
+                                                        track_view_synth_popup_cb, pcell);
+      gxk_cell_renderer_popup_dialog (pcell, dialog);
+    }
+}
+
+static void
+track_view_post_synth_popup (BstTrackView         *self,
+                             const gchar          *strpath,
+                             const gchar          *text,
+                             GxkCellRendererPopup *pcell)
+{
+  g_return_if_fail (BST_IS_TRACK_VIEW (self));
+
+  if (strpath)
+    {
+      gint row = gxk_tree_spath_index0 (strpath);
+      SfiProxy item = bst_item_view_get_proxy (BST_ITEM_VIEW (self), row);
+      BseProxySeq *pseq = bse_item_list_proxies (item, "pnet");
+      GtkWidget *dialog = bst_track_synth_dialog_popup (self, item, pseq, 0,
+                                                        track_view_synth_popup_cb, pcell);
+      gxk_cell_renderer_popup_dialog (pcell, dialog);
+    }
+}
+
 static void
 track_view_mute_toggled (BstTrackView          *self,
 			 const gchar           *strpath,
@@ -204,10 +330,7 @@ track_view_mute_toggled (BstTrackView          *self,
   if (strpath)
     {
       gint row = gxk_tree_spath_index0 (strpath);
-      guint seqid = row + 1;
-      SfiProxy item = bse_container_get_item (BST_ITEM_VIEW (self)->container,
-					      BST_ITEM_VIEW (self)->item_type,
-					      seqid);
+      SfiProxy item = bst_item_view_get_proxy (BST_ITEM_VIEW (self), row);
       if (item)
 	{
 	  gboolean muted;
@@ -229,10 +352,7 @@ track_view_voices_edited (BstTrackView *self,
   if (strpath)
     {
       gint row = gxk_tree_spath_index0 (strpath);
-      guint seqid = row + 1;
-      SfiProxy item = bse_container_get_item (BST_ITEM_VIEW (self)->container,
-					      BST_ITEM_VIEW (self)->item_type,
-					      seqid);
+      SfiProxy item = bst_item_view_get_proxy (BST_ITEM_VIEW (self), row);
       if (item)
 	{
 	  int i = strtol (text, NULL, 10);
@@ -242,56 +362,15 @@ track_view_voices_edited (BstTrackView *self,
     }
 }
 
-static void
-track_view_synth_edited (BstTrackView *self,
-			 const gchar  *strpath,
-			 const gchar  *text)
-{
-  g_return_if_fail (BST_IS_TRACK_VIEW (self));
-
-  if (strpath)
-    {
-      gint row = gxk_tree_spath_index0 (strpath);
-      guint seqid = row + 1;
-      SfiProxy snet = 0, item = bse_container_get_item (BST_ITEM_VIEW (self)->container,
-							BST_ITEM_VIEW (self)->item_type,
-							seqid);
-      guint i, l = text ? strlen (text) : 0;
-      if (l)
-	{
-	  SfiProxy ematch = 0, tmatch = 0;
-	  /* list possible snet candidates */
-	  BseProxySeq *pseq = bse_item_list_proxies (item, "snet");
-	  for (i = 0; !ematch && i < pseq->n_proxies; i++)
-	    {
-	      const gchar *path = bse_item_get_uname_path (pseq->proxies[i]);
-	      guint j = path ? strlen (path) : 0;
-	      if (j == l && strcmp (text, path) == 0)
-		ematch = pseq->proxies[i];	/* find exact match */
-	      else if (!tmatch && j >= l && strcmp (text, path + j - l) == 0)
-		tmatch = pseq->proxies[i];	/* fallback to first tail match */
-	    }
-	  snet = ematch ? ematch : tmatch;
-	}
-      bse_proxy_set (item, "snet", snet, NULL);
-    }
-}
-
 static SfiProxy
 get_track (gpointer data,
 	   gint     row)
 {
-  BstTrackView *self = BST_TRACK_VIEW (data);
-  BstItemView *iview = BST_ITEM_VIEW (self);
-  guint seqid = row + 1;
-  if (iview->container && seqid > 0)
-    return bse_container_get_item (iview->container, iview->item_type, seqid);
-  return 0;
+  return bst_item_view_get_proxy (BST_ITEM_VIEW (data), row);
 }
 
 static void
-selection_changed (BstTrackView     *self,
-		   GtkTreeSelection *tsel)
+update_selection (BstTrackView     *self)
 {
   BstItemView *iview = BST_ITEM_VIEW (self);
   if (self->troll && iview->tree)
@@ -347,18 +426,24 @@ bst_track_view_create_tree (BstItemView *iview)
   GtkObject *adjustment;
   GtkPaned *paned;
   GtkTreeSelection *tsel;
+  GtkTreeModel *smodel;
+  GxkListWrapper *lwrapper;
 
   /* item list model */
-  iview->wlist = gxk_list_wrapper_new (N_COLS,
-				       G_TYPE_STRING,	/* COL_NAME */
-				       G_TYPE_BOOLEAN,	/* COL_MUTE */
-				       G_TYPE_STRING,	/* COL_VOICES */
-				       G_TYPE_STRING,	/* COL_SYNTH */
-				       G_TYPE_STRING	/* COL_BLURB */
-				       );
-  g_signal_connect_object (iview->wlist, "fill-value",
+  lwrapper = gxk_list_wrapper_new (N_COLS,
+				   G_TYPE_STRING,	/* COL_SEQID */
+				   G_TYPE_STRING,	/* COL_NAME */
+				   G_TYPE_BOOLEAN,	/* COL_MUTE */
+				   G_TYPE_STRING,	/* COL_VOICES */
+				   G_TYPE_STRING,	/* COL_SYNTH */
+				   G_TYPE_STRING,	/* COL_POST_SYNTH */
+				   G_TYPE_STRING	/* COL_BLURB */
+				   );
+  smodel = bst_item_view_adapt_list_wrapper (iview, lwrapper);
+  g_signal_connect_object (lwrapper, "fill-value",
 			   G_CALLBACK (track_view_fill_value),
 			   iview, G_CONNECT_SWAPPED);
+  g_object_unref (lwrapper);
   
   /* scrollbars */
   hscroll1 = gtk_hscrollbar_new (NULL);
@@ -371,7 +456,7 @@ bst_track_view_create_tree (BstItemView *iview)
   tlframe = g_object_new (GTK_TYPE_FRAME, "shadow_type", GTK_SHADOW_IN, NULL);
   iview->tree = g_object_new (GTK_TYPE_TREE_VIEW,
 			      "can_focus", TRUE,
-			      "model", iview->wlist,
+			      "model", smodel,
 			      "parent", tlframe,
 			      "rules_hint", TRUE,
 			      "height_request", BST_ITEM_VIEW_TREE_HEIGHT,
@@ -382,7 +467,8 @@ bst_track_view_create_tree (BstItemView *iview)
   gtk_tree_view_set_vadjustment (iview->tree, gtk_range_get_adjustment (GTK_RANGE (vscroll)));
   tsel = gtk_tree_view_get_selection (iview->tree);
   gtk_tree_selection_set_mode (tsel, GTK_SELECTION_BROWSE);
-  gxk_tree_selection_force_browse (tsel, GTK_TREE_MODEL (iview->wlist));
+  gxk_tree_selection_force_browse (tsel, smodel);
+  g_object_unref (smodel);
 
   /* track roll */
   trframe = g_object_new (GTK_TYPE_FRAME, "shadow_type", GTK_SHADOW_IN, NULL);
@@ -397,8 +483,8 @@ bst_track_view_create_tree (BstItemView *iview)
   
   /* link track roll to tree view and list model */
   g_signal_connect_object (tsel, "changed",
-			   G_CALLBACK (selection_changed),
-			   self, G_CONNECT_SWAPPED);
+			   G_CALLBACK (update_selection),
+			   self, G_CONNECT_SWAPPED | G_CONNECT_AFTER);
   bst_track_roll_set_size_callbacks (self->troll,
 				     iview->tree,
 				     gxk_tree_view_get_bin_window_pos,
@@ -407,6 +493,9 @@ bst_track_view_create_tree (BstItemView *iview)
   g_signal_connect_object (iview->tree, "size_allocate",
 			   G_CALLBACK (bst_track_roll_reallocate),
 			   self->troll, G_CONNECT_SWAPPED | G_CONNECT_AFTER);
+  g_signal_connect_object (iview->tree, "size_allocate",
+			   G_CALLBACK (update_selection),
+			   self, G_CONNECT_SWAPPED | G_CONNECT_AFTER);
   g_signal_connect_object (self->troll, "select-row",
 			   G_CALLBACK (gxk_tree_view_focus_row),
 			   iview->tree, G_CONNECT_SWAPPED);
@@ -487,6 +576,7 @@ bst_track_view_create_tree (BstItemView *iview)
   paned = g_object_new (GTK_TYPE_HPANED,
 			"height_request", 120,
 			"border_width", 0,
+			"position", 240,
 			NULL);
   gtk_paned_pack1 (paned, vb1, FALSE, TRUE);
   gtk_paned_pack2 (paned, vb2, TRUE, TRUE);
@@ -497,20 +587,27 @@ bst_track_view_create_tree (BstItemView *iview)
   gtk_box_pack_start (GTK_BOX (hb), vb3, FALSE, TRUE, 0);
 
   /* add list view columns */
-  gxk_tree_view_add_text_column (iview->tree,
-				 COL_NAME, 0.0, "Name", NULL,
+  if (BST_DVL_HINTS)
+    gxk_tree_view_add_text_column (iview->tree, COL_SEQID, "S",
+                                   0.0, "ID", NULL,
+                                   NULL, NULL, 0);
+  gxk_tree_view_add_text_column (iview->tree, COL_NAME, "S",
+				 0.0, "Name", NULL,
 				 bst_item_view_name_edited, self, G_CONNECT_SWAPPED);
-  gxk_tree_view_add_toggle_column (iview->tree,
-				   COL_MUTE, 0.5, "M", "Notes from unchecked tracks are muted during playback",
+  gxk_tree_view_add_toggle_column (iview->tree, COL_MUTE, "",
+				   0.5, "M", "Notes from unchecked tracks are muted during playback",
 				   track_view_mute_toggled, self, G_CONNECT_SWAPPED);
-  gxk_tree_view_add_text_column (iview->tree,
-				 COL_VOICES, 0.5, "V", "Maximum number of voices for simultaneous playback",
+  gxk_tree_view_add_text_column (iview->tree, COL_VOICES, "",
+				 0.5, "V", "Maximum number of voices for simultaneous playback",
 				 track_view_voices_edited, self, G_CONNECT_SWAPPED);
-  gxk_tree_view_add_popup_column (iview->tree,
-				  COL_SYNTH, 0.5, "Synth", "Synthesizer network to be used by this track",
-				  track_view_synth_edited, self, G_CONNECT_SWAPPED);
-  gxk_tree_view_add_text_column (iview->tree,
-				 COL_BLURB, 0.0, "Comment", NULL,
+  gxk_tree_view_add_popup_column (iview->tree, COL_SYNTH, "#",
+				  0.5, "Synth", "Synthesizer network or wave to be used by this track",
+				  track_view_synth_edited, track_view_synth_popup, self, G_CONNECT_SWAPPED);
+  gxk_tree_view_add_popup_column (iview->tree, COL_POST_SYNTH, "#",
+				  0.5, "Post", "Postprocessing Synthesizer network for this track",
+				  track_view_post_synth_edited, track_view_post_synth_popup, self, G_CONNECT_SWAPPED);
+  gxk_tree_view_add_text_column (iview->tree, COL_BLURB, "",
+				 0.0, "Comment", NULL,
 				 bst_item_view_blurb_edited, self, G_CONNECT_SWAPPED);
 
   /* make widgets visible */
@@ -523,7 +620,7 @@ track_changed (SfiProxy      track,
 {
   if (self->troll)
     {
-      gint row = bse_item_get_seqid (track) - 1;
+      gint row = bst_item_view_get_proxy_row (BST_ITEM_VIEW (self), track);
       bst_track_roll_queue_draw_row (self->troll, row);
     }
 }
@@ -545,8 +642,6 @@ track_view_set_container (BstItemView *iview,
     bse_proxy_disconnect (iview->container,
 			  "any_signal", track_view_pointer_changed, self,
 			  "any_signal", track_view_marks_changed, self,
-			  "any_signal", track_view_marks_changed, self,
-			  "any_signal", track_view_marks_changed, self,
 			  "any_signal", track_view_repeat_changed, self,
 			  NULL);
   BST_ITEM_VIEW_CLASS (parent_class)->set_container (iview, new_container);
@@ -564,6 +659,14 @@ track_view_set_container (BstItemView *iview,
     }
 }
 
+static void
+track_property_changed (SfiProxy     item,
+                        const gchar *property_name,
+                        BstItemView *iview)
+{
+  bst_item_view_refresh (iview, item);
+}
+
 static void
 track_view_listen_on (BstItemView *iview,
 		      SfiProxy     item)
@@ -572,6 +675,15 @@ track_view_listen_on (BstItemView *iview,
   bse_proxy_connect (item,
 		     "signal::changed", track_changed, iview,
 		     NULL);
+  bse_proxy_connect (item,
+                     /* COL_SEQID handled by GxkListWrapper */
+                     /* COL_NAME handled by GxkListWrapper */
+                     "signal::property-notify::muted", track_property_changed, iview, /* COL_MUTE */
+                     "signal::property-notify::n_voices", track_property_changed, iview, /* COL_VOICES */
+                     "signal::property-notify::snet", track_property_changed, iview, /* COL_SYNTH */
+                     "signal::property-notify::pnet", track_property_changed, iview, /* COL_POST_SYNTH */
+                     /* COL_BLURB handled by GxkListWrapper */
+                     NULL);
 }
 
 static void
@@ -580,57 +692,55 @@ track_view_unlisten_on (BstItemView *iview,
 {
   bse_proxy_disconnect (item,
 			"any_signal", track_changed, iview,
+			"any_signal", track_property_changed, iview,
 			NULL);
   BST_ITEM_VIEW_CLASS (parent_class)->unlisten_on (iview, item);
 }
 
-void
-bst_track_view_operate (BstItemView *item_view,
-			BstOps       op)
+static void
+bst_track_view_activate (BstActivatable         *activatable,
+                         gulong                  action)
 {
-  BstTrackView *track_view = BST_TRACK_VIEW (item_view);
+  BstTrackView *self = BST_TRACK_VIEW (activatable);
+  BstItemView *item_view = BST_ITEM_VIEW (self);
   SfiProxy song = item_view->container;
 
-  g_return_if_fail (bst_track_view_can_operate (item_view, op));
-
-  switch (op)
+  switch (action)
     {
       SfiProxy item;
-    case BST_OP_TRACK_ADD:
+    case BST_ACTION_ADD_TRACK:
+      bse_item_group_undo (song, "Add Track");
       item = bse_song_create_track (song);
       if (item)
 	{
 	  gchar *string = g_strdup_printf ("Track-%02X", bse_item_get_seqid (item));
-	  bse_proxy_set (item, "uname", string, NULL);
+	  bse_item_set_name (item, string);
 	  g_free (string);
 	  bst_item_view_select (item_view, item);
 	}
+      bse_item_ungroup_undo (song);
       break;
-    case BST_OP_TRACK_DELETE:
+    case BST_ACTION_DELETE_TRACK:
       item = bst_item_view_get_current (item_view);
       bse_song_remove_track (song, item);
       break;
-    default:
-      break;
     }
-  
-  bst_update_can_operate (GTK_WIDGET (track_view));
+  bst_widget_update_activatable (activatable);
 }
 
-gboolean
-bst_track_view_can_operate (BstItemView *item_view,
-			    BstOps	   op)
+static gboolean
+bst_track_view_can_activate (BstActivatable *activatable,
+                             gulong          action)
 {
-  BstTrackView *track_view = BST_TRACK_VIEW (item_view);
+  BstTrackView *self = BST_TRACK_VIEW (activatable);
+  BstItemView *item_view = BST_ITEM_VIEW (self);
   
-  g_return_val_if_fail (BST_IS_TRACK_VIEW (track_view), FALSE);
-  
-  switch (op)
+  switch (action)
     {
       SfiProxy item;
-    case BST_OP_TRACK_ADD:
+    case BST_ACTION_ADD_TRACK:
       return TRUE;
-    case BST_OP_TRACK_DELETE:
+    case BST_ACTION_DELETE_TRACK:
       item = bst_item_view_get_current (item_view);
       return item != 0;
     default:
