@@ -31,12 +31,6 @@
 
 /* --- prototypes --- */
 static void	_bst_init_idl			(void);
-static void	traverse_viewable_changed	(GtkWidget	*widget,
-						 gpointer	 data);
-
-
-/* --- variables --- */
-static gulong viewable_changed_id = 0;
 
 
 /* --- functions --- */
@@ -47,15 +41,6 @@ _bst_init_utils (void)
   
   g_assert (initialized++ == 0);
 
-  /* Gtk+ patchups */
-  viewable_changed_id = g_signal_newv ("viewable-changed",
-				       G_TYPE_FROM_CLASS (gtk_type_class (GTK_TYPE_WIDGET)),
-				       G_SIGNAL_RUN_LAST,
-				       g_cclosure_new (G_CALLBACK (traverse_viewable_changed), NULL, NULL),
-				       NULL, NULL,
-				       gtk_marshal_VOID__VOID,
-				       G_TYPE_NONE, 0, NULL);
-
   /* initialize generated type ids */
   {
     static struct {
@@ -202,18 +187,20 @@ bst_window_sync_title_to_proxy (gpointer     window,
 				SfiProxy     proxy,
 				const gchar *title_format)
 {
+  gchar *p;
+
   g_return_if_fail (GTK_IS_WINDOW (window));
   if (proxy)
     {
       g_return_if_fail (BSE_IS_ITEM (proxy));
       g_return_if_fail (title_format != NULL);
-      g_return_if_fail (strstr (title_format, "%s") != NULL);
+      /* g_return_if_fail (strstr (title_format, "%s") != NULL); */
     }
 
-  if (proxy)
+  p = title_format ? strstr (title_format, "%s") : NULL;
+  if (proxy && p)
     {
       TitleSync *tsync = g_new0 (TitleSync, 1);
-      gchar *p = strstr (title_format, "%s");
 
       tsync->window = window;
       tsync->proxy = proxy;
@@ -227,47 +214,12 @@ bst_window_sync_title_to_proxy (gpointer     window,
     }
   else
     {
-      if (g_object_get_data (window, "bst-title-sync"))
-	{
-	  g_object_set (window, "title", NULL, NULL);
-	  g_object_set_data (window, "bst-title-sync", NULL);
-	}
+      g_object_set_data (window, "bst-title-sync", NULL);
+      g_object_set (window, "title", title_format, NULL);
     }
 }
 
 
-/* --- Gtk+ Utilities --- */
-void
-gtk_widget_viewable_changed (GtkWidget *widget)
-{
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-
-  g_signal_emit (widget, viewable_changed_id, 0);
-}
-
-static void
-traverse_viewable_changed (GtkWidget *widget,
-			   gpointer   data)
-{
-  if (GTK_IS_CONTAINER (widget))
-    gtk_container_forall (GTK_CONTAINER (widget), (GtkCallback) gtk_widget_viewable_changed, NULL);
-}
-
-gboolean
-gtk_widget_viewable (GtkWidget *widget)
-{
-  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
-
-  while (widget)
-    {
-      if (!GTK_WIDGET_MAPPED (widget))
-	return FALSE;
-      widget = widget->parent;
-    }
-  return TRUE;
-}
-
-
 /* --- field mask --- */
 static GQuark gmask_quark = 0;
 typedef struct {
@@ -329,7 +281,7 @@ gmask_form (GtkWidget   *parent,
   gmask->parent = g_object_ref (parent);
   gtk_object_sink (GTK_OBJECT (parent));
   gmask->action = action;
-  gpack = CLAMP (gpack, BST_GMASK_FIT, BST_GMASK_BIG);
+  gpack = CLAMP (gpack, BST_GMASK_FIT, BST_GMASK_CENTER);
   gmask->gpack = gpack;
 
   return action;
@@ -388,6 +340,8 @@ bst_gmask_container_create (guint    border_width,
  @* %BST_GMASK_INTERLEAVE - allow the action widget to expand across auxillary
  * columns if it requests that much space,
  @* %BST_GMASK_BIG - force expansion of the action widget across all possible
+ * columns up to the prompt,
+ @* %BST_GMASK_CENTER - center the action widget within space across all possible
  * columns up to the prompt.
  */
 BstGMask*
@@ -922,7 +876,7 @@ bst_gmask_pack (BstGMask *mask)
 	gtk_box_pack_end (GTK_BOX (action), atail, FALSE, TRUE, 0);
     }
   n = c;
-  if (gmask->gpack == BST_GMASK_BIG ||
+  if (gmask->gpack == BST_GMASK_BIG || gmask->gpack == BST_GMASK_CENTER ||
       gmask->gpack == BST_GMASK_INTERLEAVE)	/* extend action to the left when possible */
     {
       if (!aux3)
@@ -931,7 +885,8 @@ bst_gmask_pack (BstGMask *mask)
 	  if (!aux2)
 	    {
 	      n--;
-	      if (gmask->gpack == BST_GMASK_BIG && !aux1)
+	      if (!aux1 && (gmask->gpack == BST_GMASK_BIG ||
+                            gmask->gpack == BST_GMASK_CENTER))
 		{
 		  n--;
 		  if (!prompt)
@@ -949,6 +904,15 @@ bst_gmask_pack (BstGMask *mask)
 			     "xscale", 0.0,
 			     "yscale", 0.0,
 			     NULL);
+  else if (gmask->gpack == BST_GMASK_CENTER)
+    action = gtk_widget_new (GTK_TYPE_ALIGNMENT,
+                             "visible", TRUE,
+                             "child", action,
+                             "xalign", 0.5,
+                             "yalign", 0.5,
+                             "xscale", 0.0,
+                             "yscale", 0.0,
+                             NULL);
   gtk_table_attach (table, action,
 		    n, c + 1, row, row + 1,
 		    GTK_SHRINK | GTK_FILL,
