@@ -33,9 +33,13 @@ extern "C" {
 #endif /* __cplusplus */
 
 
-/* --- some globally used defines --- */
+/* --- some globally used macros --- */
 #define	_(x)	(x)
 #define	N_(x)	(x)
+#define BSE_VERSION_CMP(v1_major, v1_minor, v1_micro, v2_major, v2_minor, v2_micro) ( \
+                                    (v1_major != v2_major) ? (v1_major > v2_major ? +1 : -1) : \
+                                    (v1_minor != v2_minor) ? (v1_minor > v2_minor ? +1 : -1) : \
+                                    (v1_micro < v2_micro ? -1 : v1_micro > v2_micro))
 
 
 /* --- BSE objects, classes & interfaces --- */
@@ -47,6 +51,10 @@ typedef struct  _BseContainer           BseContainer;
 typedef struct  _BseContainerClass      BseContainerClass;
 typedef struct  _BseContextMerger       BseContextMerger;
 typedef struct  _BseContextMergerClass  BseContextMergerClass;
+typedef struct  _BseCSynth              BseCSynth;
+typedef struct  _BseCSynthClass         BseCSynthClass;
+typedef struct  _BseEditableSample      BseEditableSample;
+typedef struct  _BseEditableSampleClass BseEditableSampleClass;
 typedef struct  _BseEffect              BseEffect;
 typedef struct  _BseEffectClass         BseEffectClass;
 typedef struct  _BseItem                BseItem;
@@ -63,6 +71,8 @@ typedef struct  _BseObject              BseObject;
 typedef struct  _BseObjectClass         BseObjectClass;
 typedef struct  _BsePart		BsePart;
 typedef struct  _BsePartClass		BsePartClass;
+typedef struct  _BsePcmWriter           BsePcmWriter;
+typedef struct  _BsePcmWriterClass      BsePcmWriterClass;
 typedef struct  _BseProcedureClass      BseProcedureClass;
 typedef struct  _BseProject             BseProject;
 typedef struct  _BseProjectClass        BseProjectClass;
@@ -85,6 +95,8 @@ typedef struct  _BseSuper               BseSuper;
 typedef struct  _BseSuperClass          BseSuperClass;
 typedef struct  _BseTrack               BseTrack;
 typedef struct  _BseTrackClass          BseTrackClass;
+typedef struct  _BseUndoStack		BseUndoStack;
+typedef struct  _BseUndoStep            BseUndoStep;
 typedef struct  _BseVoice		BseVoice;
 typedef struct  _BseWave                BseWave;
 typedef struct  _BseWaveRepo            BseWaveRepo;
@@ -100,10 +112,6 @@ typedef struct  _BsePluginClass         BsePluginClass;
 
 /* --- anticipated enums --- */
 typedef enum                    /*< skip >*/
-{
-  BSE_TOKEN_UNMATCHED           = G_TOKEN_LAST + 1
-} BseTokenType;
-typedef enum                    /*< skip >*/
 {
   BSE_PIXDATA_RGB               = 3,
   BSE_PIXDATA_RGBA              = 4,
@@ -139,8 +147,9 @@ struct _BsePixdata
 typedef void          (*BseFunc)             (void);
 typedef void          (*BseIOWatch)	     (gpointer		 data,
 					      GPollFD		*pfd);
-typedef BseTokenType  (*BseTryStatement)     (gpointer           func_data,
+typedef SfiTokenType  (*BseTryStatement)     (gpointer           context_data,
                                               BseStorage        *storage,
+                                              GScanner          *scanner,
                                               gpointer           user_data);
 typedef BseObject*    (*BseUPathResolver)    (gpointer           func_data,
                                               GType              required_type,
