@@ -4,6 +4,7 @@
  **/
   { &bse_type_builtin_id_BseObject, bse_type_builtin_register_BseObject },
   { &bse_type_builtin_id_BseItem, bse_type_builtin_register_BseItem },
+  { &bse_type_builtin_id_BsePcmWriter, bse_type_builtin_register_BsePcmWriter },
   { &bse_type_builtin_id_BseSource, bse_type_builtin_register_BseSource },
   { &bse_type_builtin_id_BseSnooper, bse_type_builtin_register_BseSnooper },
   { &bse_type_builtin_id_BseConstant, bse_type_builtin_register_BseConstant },
@@ -19,11 +20,11 @@
   { &bse_type_builtin_id_BseContextMerger, bse_type_builtin_register_BseContextMerger },
   { &bse_type_builtin_id_BseSubIPort, bse_type_builtin_register_BseSubIPort },
   { &bse_type_builtin_id_BsePlugin, bse_type_builtin_register_BsePlugin },
+  { &bse_type_builtin_id_BseCSynth, bse_type_builtin_register_BseCSynth },
   { &bse_type_builtin_id_BseMidiDevice, bse_type_builtin_register_BseMidiDevice },
   { &bse_type_builtin_id_BseMidiDeviceNULL, bse_type_builtin_register_BseMidiDeviceNULL },
   { &bse_type_builtin_id_BseSong, bse_type_builtin_register_BseSong },
   { &bse_type_builtin_id_BseBiquadFilter, bse_type_builtin_register_BseBiquadFilter },
-  { &bse_type_builtin_id_BseBinData, bse_type_builtin_register_BseBinData },
   { &bse_type_builtin_id_BseMidiNotifier, bse_type_builtin_register_BseMidiNotifier },
   { &bse_type_builtin_id_BseProject, bse_type_builtin_register_BseProject },
   { &bse_type_builtin_id_BseTrack, bse_type_builtin_register_BseTrack },
