@@ -33,8 +33,6 @@ extern GType BSE_TYPE_ID (BseTimeRangeType);
 /* --- bsemidievent.h --- */
 #define BSE_TYPE_MIDI_EVENT_TYPE	    (BSE_TYPE_ID (BseMidiEventType))
 extern GType BSE_TYPE_ID (BseMidiEventType);
-#define BSE_TYPE_MIDI_SIGNAL_TYPE	    (BSE_TYPE_ID (BseMidiSignalType))
-extern GType BSE_TYPE_ID (BseMidiSignalType);
 
 /* --- bseproject.h --- */
 #define BSE_TYPE_PROJECT_STATE	    (BSE_TYPE_ID (BseProjectState))
@@ -206,10 +204,12 @@ typedef struct _BseNoteSeq BseNoteSeq;
 typedef struct _BseStringSeq BseStringSeq;
 typedef struct _BseProxySeq BseProxySeq;
 typedef struct _BsePartNoteSeq BsePartNoteSeq;
+typedef struct _BsePartControlSeq BsePartControlSeq;
 typedef struct _BseCategorySeq BseCategorySeq;
 typedef struct _BseDotSeq BseDotSeq;
 typedef struct _BseTrackPartSeq BseTrackPartSeq;
 typedef struct _BsePartNote BsePartNote;
+typedef struct _BsePartControl BsePartControl;
 typedef struct _BseNoteDescription BseNoteDescription;
 typedef struct _BseNoteSequence BseNoteSequence;
 typedef struct _BseIcon BseIcon;
@@ -220,6 +220,183 @@ typedef struct _BseSongTiming BseSongTiming;
 typedef struct _BseSampleFileInfo BseSampleFileInfo;
 typedef struct _BseGConfig BseGConfig;
 
+typedef enum {
+  BSE_MIDI_SIGNAL_PROGRAM = 1,
+  BSE_MIDI_SIGNAL_PRESSURE = 2,
+  BSE_MIDI_SIGNAL_PITCH_BEND = 3,
+  BSE_MIDI_SIGNAL_VELOCITY = 4,
+  BSE_MIDI_SIGNAL_FINE_TUNE = 5,
+  BSE_MIDI_SIGNAL_CONTINUOUS_0 = 64,
+  BSE_MIDI_SIGNAL_CONTINUOUS_1 = 65,
+  BSE_MIDI_SIGNAL_CONTINUOUS_2 = 66,
+  BSE_MIDI_SIGNAL_CONTINUOUS_3 = 67,
+  BSE_MIDI_SIGNAL_CONTINUOUS_4 = 68,
+  BSE_MIDI_SIGNAL_CONTINUOUS_5 = 69,
+  BSE_MIDI_SIGNAL_CONTINUOUS_6 = 70,
+  BSE_MIDI_SIGNAL_CONTINUOUS_7 = 71,
+  BSE_MIDI_SIGNAL_CONTINUOUS_8 = 72,
+  BSE_MIDI_SIGNAL_CONTINUOUS_9 = 73,
+  BSE_MIDI_SIGNAL_CONTINUOUS_10 = 74,
+  BSE_MIDI_SIGNAL_CONTINUOUS_11 = 75,
+  BSE_MIDI_SIGNAL_CONTINUOUS_12 = 76,
+  BSE_MIDI_SIGNAL_CONTINUOUS_13 = 77,
+  BSE_MIDI_SIGNAL_CONTINUOUS_14 = 78,
+  BSE_MIDI_SIGNAL_CONTINUOUS_15 = 79,
+  BSE_MIDI_SIGNAL_CONTINUOUS_16 = 80,
+  BSE_MIDI_SIGNAL_CONTINUOUS_17 = 81,
+  BSE_MIDI_SIGNAL_CONTINUOUS_18 = 82,
+  BSE_MIDI_SIGNAL_CONTINUOUS_19 = 83,
+  BSE_MIDI_SIGNAL_CONTINUOUS_20 = 84,
+  BSE_MIDI_SIGNAL_CONTINUOUS_21 = 85,
+  BSE_MIDI_SIGNAL_CONTINUOUS_22 = 86,
+  BSE_MIDI_SIGNAL_CONTINUOUS_23 = 87,
+  BSE_MIDI_SIGNAL_CONTINUOUS_24 = 88,
+  BSE_MIDI_SIGNAL_CONTINUOUS_25 = 89,
+  BSE_MIDI_SIGNAL_CONTINUOUS_26 = 90,
+  BSE_MIDI_SIGNAL_CONTINUOUS_27 = 91,
+  BSE_MIDI_SIGNAL_CONTINUOUS_28 = 92,
+  BSE_MIDI_SIGNAL_CONTINUOUS_29 = 93,
+  BSE_MIDI_SIGNAL_CONTINUOUS_30 = 94,
+  BSE_MIDI_SIGNAL_CONTINUOUS_31 = 95,
+  BSE_MIDI_SIGNAL_CONSTANT_HIGH = 96,
+  BSE_MIDI_SIGNAL_CONSTANT_CENTER = 97,
+  BSE_MIDI_SIGNAL_CONSTANT_LOW = 98,
+  BSE_MIDI_SIGNAL_CONSTANT_NEGATIVE_CENTER = 99,
+  BSE_MIDI_SIGNAL_CONSTANT_NEGATIVE_HIGH = 100,
+  BSE_MIDI_SIGNAL_PARAMETER = 101,
+  BSE_MIDI_SIGNAL_NON_PARAMETER = 102,
+  BSE_MIDI_SIGNAL_CONTROL_0 = 128,
+  BSE_MIDI_SIGNAL_CONTROL_1 = 129,
+  BSE_MIDI_SIGNAL_CONTROL_2 = 130,
+  BSE_MIDI_SIGNAL_CONTROL_3 = 131,
+  BSE_MIDI_SIGNAL_CONTROL_4 = 132,
+  BSE_MIDI_SIGNAL_CONTROL_5 = 133,
+  BSE_MIDI_SIGNAL_CONTROL_6 = 134,
+  BSE_MIDI_SIGNAL_CONTROL_7 = 135,
+  BSE_MIDI_SIGNAL_CONTROL_8 = 136,
+  BSE_MIDI_SIGNAL_CONTROL_9 = 137,
+  BSE_MIDI_SIGNAL_CONTROL_10 = 138,
+  BSE_MIDI_SIGNAL_CONTROL_11 = 139,
+  BSE_MIDI_SIGNAL_CONTROL_12 = 140,
+  BSE_MIDI_SIGNAL_CONTROL_13 = 141,
+  BSE_MIDI_SIGNAL_CONTROL_14 = 142,
+  BSE_MIDI_SIGNAL_CONTROL_15 = 143,
+  BSE_MIDI_SIGNAL_CONTROL_16 = 144,
+  BSE_MIDI_SIGNAL_CONTROL_17 = 145,
+  BSE_MIDI_SIGNAL_CONTROL_18 = 146,
+  BSE_MIDI_SIGNAL_CONTROL_19 = 147,
+  BSE_MIDI_SIGNAL_CONTROL_20 = 148,
+  BSE_MIDI_SIGNAL_CONTROL_21 = 149,
+  BSE_MIDI_SIGNAL_CONTROL_22 = 150,
+  BSE_MIDI_SIGNAL_CONTROL_23 = 151,
+  BSE_MIDI_SIGNAL_CONTROL_24 = 152,
+  BSE_MIDI_SIGNAL_CONTROL_25 = 153,
+  BSE_MIDI_SIGNAL_CONTROL_26 = 154,
+  BSE_MIDI_SIGNAL_CONTROL_27 = 155,
+  BSE_MIDI_SIGNAL_CONTROL_28 = 156,
+  BSE_MIDI_SIGNAL_CONTROL_29 = 157,
+  BSE_MIDI_SIGNAL_CONTROL_30 = 158,
+  BSE_MIDI_SIGNAL_CONTROL_31 = 159,
+  BSE_MIDI_SIGNAL_CONTROL_32 = 160,
+  BSE_MIDI_SIGNAL_CONTROL_33 = 161,
+  BSE_MIDI_SIGNAL_CONTROL_34 = 162,
+  BSE_MIDI_SIGNAL_CONTROL_35 = 163,
+  BSE_MIDI_SIGNAL_CONTROL_36 = 164,
+  BSE_MIDI_SIGNAL_CONTROL_37 = 165,
+  BSE_MIDI_SIGNAL_CONTROL_38 = 166,
+  BSE_MIDI_SIGNAL_CONTROL_39 = 167,
+  BSE_MIDI_SIGNAL_CONTROL_40 = 168,
+  BSE_MIDI_SIGNAL_CONTROL_41 = 169,
+  BSE_MIDI_SIGNAL_CONTROL_42 = 170,
+  BSE_MIDI_SIGNAL_CONTROL_43 = 171,
+  BSE_MIDI_SIGNAL_CONTROL_44 = 172,
+  BSE_MIDI_SIGNAL_CONTROL_45 = 173,
+  BSE_MIDI_SIGNAL_CONTROL_46 = 174,
+  BSE_MIDI_SIGNAL_CONTROL_47 = 175,
+  BSE_MIDI_SIGNAL_CONTROL_48 = 176,
+  BSE_MIDI_SIGNAL_CONTROL_49 = 177,
+  BSE_MIDI_SIGNAL_CONTROL_50 = 178,
+  BSE_MIDI_SIGNAL_CONTROL_51 = 179,
+  BSE_MIDI_SIGNAL_CONTROL_52 = 180,
+  BSE_MIDI_SIGNAL_CONTROL_53 = 181,
+  BSE_MIDI_SIGNAL_CONTROL_54 = 182,
+  BSE_MIDI_SIGNAL_CONTROL_55 = 183,
+  BSE_MIDI_SIGNAL_CONTROL_56 = 184,
+  BSE_MIDI_SIGNAL_CONTROL_57 = 185,
+  BSE_MIDI_SIGNAL_CONTROL_58 = 186,
+  BSE_MIDI_SIGNAL_CONTROL_59 = 187,
+  BSE_MIDI_SIGNAL_CONTROL_60 = 188,
+  BSE_MIDI_SIGNAL_CONTROL_61 = 189,
+  BSE_MIDI_SIGNAL_CONTROL_62 = 190,
+  BSE_MIDI_SIGNAL_CONTROL_63 = 191,
+  BSE_MIDI_SIGNAL_CONTROL_64 = 192,
+  BSE_MIDI_SIGNAL_CONTROL_65 = 193,
+  BSE_MIDI_SIGNAL_CONTROL_66 = 194,
+  BSE_MIDI_SIGNAL_CONTROL_67 = 195,
+  BSE_MIDI_SIGNAL_CONTROL_68 = 196,
+  BSE_MIDI_SIGNAL_CONTROL_69 = 197,
+  BSE_MIDI_SIGNAL_CONTROL_70 = 198,
+  BSE_MIDI_SIGNAL_CONTROL_71 = 199,
+  BSE_MIDI_SIGNAL_CONTROL_72 = 200,
+  BSE_MIDI_SIGNAL_CONTROL_73 = 201,
+  BSE_MIDI_SIGNAL_CONTROL_74 = 202,
+  BSE_MIDI_SIGNAL_CONTROL_75 = 203,
+  BSE_MIDI_SIGNAL_CONTROL_76 = 204,
+  BSE_MIDI_SIGNAL_CONTROL_77 = 205,
+  BSE_MIDI_SIGNAL_CONTROL_78 = 206,
+  BSE_MIDI_SIGNAL_CONTROL_79 = 207,
+  BSE_MIDI_SIGNAL_CONTROL_80 = 208,
+  BSE_MIDI_SIGNAL_CONTROL_81 = 209,
+  BSE_MIDI_SIGNAL_CONTROL_82 = 210,
+  BSE_MIDI_SIGNAL_CONTROL_83 = 211,
+  BSE_MIDI_SIGNAL_CONTROL_84 = 212,
+  BSE_MIDI_SIGNAL_CONTROL_85 = 213,
+  BSE_MIDI_SIGNAL_CONTROL_86 = 214,
+  BSE_MIDI_SIGNAL_CONTROL_87 = 215,
+  BSE_MIDI_SIGNAL_CONTROL_88 = 216,
+  BSE_MIDI_SIGNAL_CONTROL_89 = 217,
+  BSE_MIDI_SIGNAL_CONTROL_90 = 218,
+  BSE_MIDI_SIGNAL_CONTROL_91 = 219,
+  BSE_MIDI_SIGNAL_CONTROL_92 = 220,
+  BSE_MIDI_SIGNAL_CONTROL_93 = 221,
+  BSE_MIDI_SIGNAL_CONTROL_94 = 222,
+  BSE_MIDI_SIGNAL_CONTROL_95 = 223,
+  BSE_MIDI_SIGNAL_CONTROL_96 = 224,
+  BSE_MIDI_SIGNAL_CONTROL_97 = 225,
+  BSE_MIDI_SIGNAL_CONTROL_98 = 226,
+  BSE_MIDI_SIGNAL_CONTROL_99 = 227,
+  BSE_MIDI_SIGNAL_CONTROL_100 = 228,
+  BSE_MIDI_SIGNAL_CONTROL_101 = 229,
+  BSE_MIDI_SIGNAL_CONTROL_102 = 230,
+  BSE_MIDI_SIGNAL_CONTROL_103 = 231,
+  BSE_MIDI_SIGNAL_CONTROL_104 = 232,
+  BSE_MIDI_SIGNAL_CONTROL_105 = 233,
+  BSE_MIDI_SIGNAL_CONTROL_106 = 234,
+  BSE_MIDI_SIGNAL_CONTROL_107 = 235,
+  BSE_MIDI_SIGNAL_CONTROL_108 = 236,
+  BSE_MIDI_SIGNAL_CONTROL_109 = 237,
+  BSE_MIDI_SIGNAL_CONTROL_110 = 238,
+  BSE_MIDI_SIGNAL_CONTROL_111 = 239,
+  BSE_MIDI_SIGNAL_CONTROL_112 = 240,
+  BSE_MIDI_SIGNAL_CONTROL_113 = 241,
+  BSE_MIDI_SIGNAL_CONTROL_114 = 242,
+  BSE_MIDI_SIGNAL_CONTROL_115 = 243,
+  BSE_MIDI_SIGNAL_CONTROL_116 = 244,
+  BSE_MIDI_SIGNAL_CONTROL_117 = 245,
+  BSE_MIDI_SIGNAL_CONTROL_118 = 246,
+  BSE_MIDI_SIGNAL_CONTROL_119 = 247,
+  BSE_MIDI_SIGNAL_CONTROL_120 = 248,
+  BSE_MIDI_SIGNAL_CONTROL_121 = 249,
+  BSE_MIDI_SIGNAL_CONTROL_122 = 250,
+  BSE_MIDI_SIGNAL_CONTROL_123 = 251,
+  BSE_MIDI_SIGNAL_CONTROL_124 = 252,
+  BSE_MIDI_SIGNAL_CONTROL_125 = 253,
+  BSE_MIDI_SIGNAL_CONTROL_126 = 254,
+  BSE_MIDI_SIGNAL_CONTROL_127 = 255,
+} BseMidiSignalType;
+const gchar* bse_midi_signal_type_to_choice (BseMidiSignalType value);
+BseMidiSignalType bse_midi_signal_type_from_choice (const gchar *choice);
+
 struct _BseIntSeq {
   guint n_ints;
   SfiInt* ints;
@@ -240,6 +417,10 @@ struct _BsePartNoteSeq {
   guint n_pnotes;
   BsePartNote** pnotes;
 };
+struct _BsePartControlSeq {
+  guint n_pcontrols;
+  BsePartControl** pcontrols;
+};
 struct _BseCategorySeq {
   guint n_cats;
   BseCategory** cats;
@@ -261,6 +442,13 @@ struct _BsePartNote {
   SfiReal velocity;
   SfiBool selected;
 };
+struct _BsePartControl {
+  SfiInt id;
+  SfiInt tick;
+  BseMidiSignalType control_type;
+  SfiReal value;
+  SfiBool selected;
+};
 struct _BseNoteDescription {
   SfiInt note;
   SfiInt octave;
@@ -364,6 +552,14 @@ SfiSeq *bse_part_note_seq_to_seq (BsePartNoteSeq* seq);
 void bse_part_note_seq_resize (BsePartNoteSeq* seq, guint new_size);
 void bse_part_note_seq_free (BsePartNoteSeq* seq);
 
+BsePartControlSeq* bse_part_control_seq_new (void);
+void bse_part_control_seq_append (BsePartControlSeq* seq, BsePartControl* element);
+BsePartControlSeq* bse_part_control_seq_copy_shallow (BsePartControlSeq* seq);
+BsePartControlSeq* bse_part_control_seq_from_seq (SfiSeq *sfi_seq);
+SfiSeq *bse_part_control_seq_to_seq (BsePartControlSeq* seq);
+void bse_part_control_seq_resize (BsePartControlSeq* seq, guint new_size);
+void bse_part_control_seq_free (BsePartControlSeq* seq);
+
 BseCategorySeq* bse_category_seq_new (void);
 void bse_category_seq_append (BseCategorySeq* seq, BseCategory* element);
 BseCategorySeq* bse_category_seq_copy_shallow (BseCategorySeq* seq);
@@ -394,6 +590,12 @@ BsePartNote* bse_part_note_from_rec (SfiRec *sfi_rec);
 SfiRec *bse_part_note_to_rec (BsePartNote* rec);
 void bse_part_note_free (BsePartNote* rec);
 
+BsePartControl* bse_part_control_new (void);
+BsePartControl* bse_part_control_copy_shallow (BsePartControl* rec);
+BsePartControl* bse_part_control_from_rec (SfiRec *sfi_rec);
+SfiRec *bse_part_control_to_rec (BsePartControl* rec);
+void bse_part_control_free (BsePartControl* rec);
+
 BseNoteDescription* bse_note_description_new (void);
 BseNoteDescription* bse_note_description_copy_shallow (BseNoteDescription* rec);
 BseNoteDescription* bse_note_description_from_rec (SfiRec *sfi_rec);
@@ -449,8 +651,12 @@ SfiRec *bse_gconfig_to_rec (BseGConfig* rec);
 void bse_gconfig_free (BseGConfig* rec);
 
 
+extern SfiChoiceValues bse_midi_signal_type_values;
+extern GType BSE_TYPE_MIDI_SIGNAL_TYPE;
 extern SfiRecFields bse_part_note_fields;
 extern GType BSE_TYPE_PART_NOTE;
+extern SfiRecFields bse_part_control_fields;
+extern GType BSE_TYPE_PART_CONTROL;
 extern SfiRecFields bse_note_description_fields;
 extern GType BSE_TYPE_NOTE_DESCRIPTION;
 extern SfiRecFields bse_note_sequence_fields;
@@ -474,6 +680,7 @@ extern GType BSE_TYPE_NOTE_SEQ;
 extern GType BSE_TYPE_STRING_SEQ;
 extern GType BSE_TYPE_PROXY_SEQ;
 extern GType BSE_TYPE_PART_NOTE_SEQ;
+extern GType BSE_TYPE_PART_CONTROL_SEQ;
 extern GType BSE_TYPE_CATEGORY_SEQ;
 extern GType BSE_TYPE_DOT_SEQ;
 extern GType BSE_TYPE_TRACK_PART_SEQ;
