@@ -15,12 +15,13 @@
   (bse-item-group-undo snet "modules2grid")      ; move all children at once
   (do ((children (bse-container-list-items snet) (cdr children)))
       ((null? children))
-    (let* ((module (car children))
-	   (x (bse-item-get module "pos-x"))
-	   (y (bse-item-get module "pos-y")))
-      (if #f (display (string-append (bse-item-get-name-or-type (car children))
-				     ": " (number->string x)
-				     ", " (number->string y)
-				     "\n")))
-      (bse-source-set-pos module (round x) (round y))))
+    (let ((module (car children)))
+      (if (bse-is-source module)
+	  (let ((x (bse-item-get module "pos-x"))
+		(y (bse-item-get module "pos-y")))
+	    (if #f (display (string-append (bse-item-get-name-or-type (car children))
+					   ": " (number->string x)
+					   ", " (number->string y)
+					   "\n")))
+	    (bse-source-set-pos module (round x) (round y))))))
   (bse-item-ungroup-undo snet))
